/*
  Buffered serial for ByVac serial devices
  Copyright (c) 2011 Jim Spence.  All right reserved.
  www.byvac.com - see terms and conditions for using hardware
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/  

#ifndef _H_BSerial_h
#define _H_BSerial_h

#include <inttypes.h>

#define TIB_MAX 32      // size of Text Input Buffer

class BSerial
{
    public:
        BSerial(uint8_t receivePin, uint8_t transmitPin, uint8_t rts_pin);
        BSerial(uint8_t receivePin, uint8_t transmitPin);
        void begin(long speed);
        void setRTS(char v);
        void clear_tib();
        char cInbuf();
        char getch (void);
        void putch(uint8_t b);
        void puts(char *s);
        void printNumber(unsigned long n, uint8_t base);
        static inline void handle_interrupt();
    private:
        static BSerial *active_object;
        bool activate();
        uint8_t _receivePin;
        uint8_t _transmitPin;
        long _baudRate;
        int _bitPeriod;
        char _tib[TIB_MAX]; // input buffer
        char _cib;       // charatres in buffer
        char _Pptr, _Gptr; // put and get pointers
        char _bfullF; // buffer full flag
        char _rts_pin;
        void fill_buf();
        char read();
};


#endif