/*
  BV4108 ByVac
  Copyright (c) 2011 Jim Spence.  All right reserved.
  www.byvac.com - see terms and conditions for using hardware
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
  Release Notes:

*/  

#include "WProgram.h"
#include "bv4108.h"


// ==============================================================
// BV4108 LCD SECTION
// ==============================================================
// ==============================================================

// **************************************************************
// 
// **************************************************************
BV4108::BV4108(uint8_t rxPin, uint8_t txPin) : ASI(rxPin, txPin)
{
}

// **************************************************************
// clear screen
// **************************************************************
void BV4108::cls(char DEVadr)
{
    nss.flush();
    nss.print(DEVadr);
    nss.print("c1\r");
    wait();
}

// **************************************************************
// Sends text to lcd
// command t isdtead of d as d does not return the prompt
// **************************************************************
void BV4108::puts(char DEVadr, char *s)
{
    nss.print(DEVadr);
    nss.print("t'");
    nss.print(s);
    nss.print("'\r");
    wait();
}

// **************************************************************
// put data, no ACK is returned for this
// **************************************************************
void BV4108::putdata(char DEVadr, char data)
{
    nss.print(DEVadr);
    nss.print("d");
    nss.print(data);
    nss.print("\r");
}

// **************************************************************
// put command.
// **************************************************************
void BV4108::putcmd(char DEVadr, char data)
{
    nss.print(DEVadr);
    nss.print("c");
    nss.print(data);
    nss.print("'\r");
    wait();
}

// **************************************************************
// places cursor at specified row and column positions
// **************************************************************
void BV4108::rowcol(char DEVadr, char row, char col)
{
char b[5];
int rc;
    nss.flush();
    nss.print(DEVadr);
    nss.print("c");
    switch(row) {
        case 1: { rc = CMD_LINE1+col; break; }
        case 2: { rc = CMD_LINE2+col; break; }
        case 3: { rc = CMD_LINE3+col; break; }
        case 4: { rc = CMD_LINE4+col; break; }
        default: { rc = CMD_LINE1+col; break; }
    }
    itoa(rc,b,16);
    nss.print(b);
    nss.print("\r");
    wait();
}

// **************************************************************
// back light on off
// **************************************************************
void BV4108::bl(char DEVadr, char ton)
{
    nss.flush();
    nss.print(DEVadr);
    if(ton) nss.print("b1\r");
    else nss.print("b0\r");
}

// **************************************************************
// change sign on -- write to EEPROM
// **************************************************************
void BV4108::custom(char DEVadr, char chno, char *buf8)
{
    if(chno > 7) return;
    nss.print(DEVadr);
    nss.print("c40");   // access cgram
    nss.print(chno); // charater to define
    nss.print("\r");
    wait();
    // place in buffer
    nss.print(DEVadr);
    nss.print("d");     // data follows 
    for(char j=0;j<8;j++) 
        nss.print(*(buf8+j));
    nss.print("\r");
    delay(300); // no ack after this
    nss.flush();
    nss.print(DEVadr);
    nss.print("c80\r");   // access ddram
    wait();
}

