/*
  BV4108 ByVac
  Copyright (c) 2011 Jim Spence.  All right reserved.
  www.byvac.com - see terms and conditions for using hardware
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
  Release Notes:

*/  

#include "WProgram.h"
#include "bv4108.h"

TEST::TEST(char x, char y): BV4108(9,9)
{
}

int TEST::thetest(char x) 
{
    return x*x;
//    nss.print("fred");
}

// ==============================================================
// ASI SECTION
// ==============================================================
// ==============================================================
// **************************************************************
// constructos is justa connection to NewSoftSerial
// **************************************************************
BV4108::BV4108(uint8_t rxPin, uint8_t txPin) : nss(rxPin,txPin) //, test(2,4)
{
}

// **************************************************************
// set up baud rate, delay and ACK char
// set delay to 0 if none required and ack to 0 ditto
// **************************************************************
void BV4108::begin(long rate, char address)
{
char b[28], *bp=b;;
    nss.begin(rate);    // set up serial
    _ASIadr=address;
    nss.flush();        // needed as all sorts in buffer
    // initialise Baud rate
    nss.print("\r"); delay(10);
    nss.print("\r"); delay(10);
    nss.print("\r"); delay(10);
    delay(400);
    // 
    nss.print("\x4\r"); // invert output
    nss.flush(); // buffer
}

// **************************************************************
// wait for the '>' or time out and return gathered input
// **************************************************************
char BV4108::wait(char *inp, char max)
{
char c, count=0, *sp=inp;
int timeout=800; // about 1 second
    *sp='\0';
    while(1) {
        c = nss.read();
        if(c < 0 ) {
            timeout--;
            delay(1);
            continue;
        }
        if( c == '>') break;
        if( count++ >= max) break;
        *(sp++)=c;
    }
    *sp='\0';
    return count;    
}
// alternative, just waits for '>' but returns nothing
void BV4108::wait()
{
char c;
int timeout=800;
    while(1) {
        c = nss.read();
        if(c < 0 ) {
            timeout--;
            delay(1);
            continue;
        }
        if( c == '>') break;
    }
}

// **************************************************************
// return all devices on bus, max is the buffer size to contain 
// devices
// **************************************************************
void BV4108::devices(char *dev, char max)
{
    nss.flush();
    nss.print("\x1\r");
    wait(dev,max);
}

// **************************************************************
// changes address, the device will not respond after this unless
// begin is issued again
// changes EEPROM contents so don't use every time
// **************************************************************
void BV4108::changeaddress(char newadr)
{
    nss.flush();
    // unlock first - this command does not send back >
    nss.print(_ASIadr);
    nss.print("U\r");
    delay(100); // because there is no ack
    nss.print(_ASIadr);
    nss.print("A");
    nss.print(newadr);
    nss.print("\r");   
    wait();
    delay(50); // because this is write to EEPROM
}

// **************************************************************
// returns version as text string
// **************************************************************
void BV4108::version(char *s, char max)
{
    nss.flush();
    nss.print(_ASIadr);
    nss.print("V\r");
    wait(s,max);
}    

// ==============================================================
// BV4108 LCD SECTION
// ==============================================================
// ==============================================================

// **************************************************************
// clear screen
// **************************************************************
void BV4108::cls()
{
    nss.flush();
    nss.print(_ASIadr);
    nss.print("c1\r");
    wait();
}

// **************************************************************
// Sends text to lcd
// command t isdtead of d as d does not return the prompt
// **************************************************************
void BV4108::puts(char *s)
{
int timeout=800;
    nss.print(_ASIadr);
    nss.print("t'");
    nss.print(s);
    nss.print("'\r");
    wait();
}

// **************************************************************
// places cursor at specified row and column positions
// **************************************************************
void BV4108::rowcol(char row, char col)
{
char b[5];
int rc;
    nss.flush();
    nss.print(_ASIadr);
    nss.print("c");
    switch(row) {
        case 1: { rc = CMD_LINE1+col; break; }
        case 2: { rc = CMD_LINE2+col; break; }
        case 3: { rc = CMD_LINE3+col; break; }
        case 4: { rc = CMD_LINE4+col; break; }
        default: { rc = CMD_LINE1+col; break; }
    }
    itoa(rc,b,16);
    nss.print(b);
    nss.print("\r");
    wait();
}

// **************************************************************
// back light on off
// **************************************************************
void BV4108::bl(char ton)
{
    nss.flush();
    nss.print(_ASIadr);
    if(ton) nss.print("b1\r");
    else nss.print("b0\r");
}
