/*
  BV4205 I2C ADC
  Copyright (c) 2011 Jim Spence.  All right reserved.
  www.byvac.com - see terms and conditions for using hardware
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/  

#include "WProgram.h"
#include "bv4205.h"
 
// **************************************************************
// **************************************************************
BV4205::BV4205(char i2adr):I2c_bv(i2adr)
{
  _i2adr=i2adr;  
}

// ==============================================================
// I2C 7 Segment Display - 4 digit
// ==============================================================
// ==============================================================

// **************************************************************
// command 1
// Select channel
// **************************************************************
void BV4205::channel(char chan)
{ 
  // 
  Wire.beginTransmission(_i2adr);
  if(chan > 9) chan = 9;
  if(chan < 0) chan = 0;
  Wire.send(1);  // command
  Wire.send(chan);
  Wire.endTransmission();
} 

// **************************************************************
// command 2
// do conversion
// **************************************************************
void BV4205::convert()
{ 
  Wire.beginTransmission(_i2adr);
  Wire.send(2);  // command
  Wire.endTransmission();
} 

// **************************************************************
// command 3
// conversion status
// **************************************************************
int BV4205::done()
{ 
int rv;
  Wire.beginTransmission(_i2adr);
  Wire.send(3);  // command
  Wire.endTransmission();
  Wire.requestFrom(_i2adr, 1); // returns 1 byte
  rv=Wire.receive();
  Wire.endTransmission();
  return rv;
} 

// **************************************************************
// command 4
// conversion result
// **************************************************************
int BV4205::result()
{ 
int rv;
  Wire.beginTransmission(_i2adr);
  Wire.send(4);  // command
  Wire.endTransmission();
  return i2_16bit();
} 

// **************************************************************
// command 5
// adc enable 
// **************************************************************
void BV4205::enable(char oon)
{ 
  Wire.beginTransmission(_i2adr);
  Wire.send(5);  // command
  Wire.send(oon); // on or off if 0
  Wire.endTransmission();
} 

// **************************************************************
// command 6
// autoscan 
// **************************************************************
void BV4205::autoscan(char oon)
{ 
  Wire.beginTransmission(_i2adr);
  Wire.send(6);  // command
  Wire.send(oon); // on or off if 0
  Wire.endTransmission();
} 

// **************************************************************
// command 7
// autoread, specify channel at 0 to 9
// **************************************************************
int BV4205::autoread(char chan)
{ 
  Wire.beginTransmission(_i2adr);
  Wire.send(7);  // command
  Wire.send(chan << 1); // 2 byte per channel
  Wire.endTransmission();
  return i2_16bit();
} 

// **************************************************************
// command 8
// aquisition delay uS
// **************************************************************
void BV4205::aqdelay(char del)
{ 
  Wire.beginTransmission(_i2adr);
  Wire.send(8);  // command
  Wire.send(del); // 2 byte per channel
  Wire.endTransmission();
} 

// **************************************************************
// command 9
// autoscan read using differential, channel is 0 to 4
// **************************************************************
int BV4205::readdif(char chan)
{ 
  Wire.beginTransmission(_i2adr);
  Wire.send(9);  // command
  Wire.send(chan); // 2 byte per channel
  Wire.endTransmission();
  return i2_16bit();
} 

// **************************************************************
// command 10
// right justification 
// **************************************************************
void BV4205::right(char oon)
{ 
  Wire.beginTransmission(_i2adr);
  Wire.send(10);  // command
  Wire.send(oon); // on or off if 0
  Wire.endTransmission();
} 

// **************************************************************
// command 11
// voltage source
// **************************************************************
void BV4205::vsource(char vplus)
{ 
  Wire.beginTransmission(_i2adr);
  Wire.send(11);  // command
  Wire.send(vplus); // +V if not 0
  Wire.endTransmission();
} 

// **************************************************************
// command 12
// conversion clock time 0 to 2
// **************************************************************
void BV4205::clock(char clk)
{ 
  Wire.beginTransmission(_i2adr);
  Wire.send(12);  // command
  Wire.send(clk); // 
  Wire.endTransmission();
} 

