/*
  BV4205 I2C GPIO
  Copyright (c) 2011 Jim Spence.  All right reserved.
  www.byvac.com - see terms and conditions for using hardware
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/  

#include "WProgram.h"
#include "bv4206.h"
 
// **************************************************************
// **************************************************************
BV4206::BV4206(char i2adr):I2c_bv(i2adr)
{
  _i2adr=i2adr;  
}

// ==============================================================
// I2C GPIO I/O Section
// ==============================================================
// ==============================================================

// **************************************************************
// Set channel to ino or out, if input is set to 0 then its output
// **************************************************************
void BV4206::IO(char channel, char input)
{ 
  // set to I/O
  Wire.beginTransmission(_i2adr);
  Wire.send(1);  // command
  Wire.send(channel); 
  if(input) Wire.send(GPIO_IN);
  else Wire.send(GPIO_OUT); 
  Wire.endTransmission();
} 

// **************************************************************
// Set pin for channel, value is 1 or 0
// **************************************************************
void BV4206::pin(char channel, char value)
{ 
  Wire.beginTransmission(_i2adr);
  Wire.send(2);  // command
  Wire.send(channel);
  if(value) Wire.send(1);
  else Wire.send(0);
  Wire.endTransmission();
} 

// **************************************************************
// Get pin for channel
// **************************************************************
int BV4206::pin(char channel)
{ 
int rv;
  Wire.beginTransmission(_i2adr);
  Wire.send(3);  // command
  Wire.send(channel);
  Wire.endTransmission();
  Wire.requestFrom(_i2adr, 1); // returns 1 byte
  rv=Wire.receive();
  Wire.endTransmission();
  return rv;
} 

// **************************************************************
// command 4, enables, disables weak pull ups, see datasheet
// **************************************************************
void BV4206::pullups(char enable)
{ 
  Wire.beginTransmission(_i2adr);
  Wire.send(4);  // command
  if(enable) Wire.send(1);
  else Wire.send(0);
  Wire.endTransmission();
} 

// ==============================================================
// I2C Timer Section
// ==============================================================
// ==============================================================

// **************************************************************
// command 10
// Output to pin 6, 1 turns it on
// **************************************************************
void BV4206::timerOut6(char onn)
{ 
  Wire.beginTransmission(_i2adr);
  Wire.send(0x10);  // command
  if(onn) Wire.send(TMR_PIN6ON);
  else Wire.send(TMR_PIN6OFF);
  Wire.endTransmission();
} 

// **************************************************************
// command 11
// prescale value, 0 to 8
// **************************************************************
void BV4206::timerPrescale(char prescale)
{ 
  Wire.beginTransmission(_i2adr);
  if(prescale > 8) prescale = 8;
  if(prescale < 0) prescale = 0;
  Wire.send(0x11);  // command
  Wire.send(prescale); // 
  Wire.endTransmission();
} 

// **************************************************************
// command 12
// timer value, 0 to 230
// **************************************************************
void BV4206::timerValue(char timer)
{ 
  Wire.beginTransmission(_i2adr);
  if(timer > 230) timer = 230;
  if(timer < 0) timer = 0;
  Wire.send(0x12);  // command
  Wire.send(timer); // 
  Wire.endTransmission();
} 

// **************************************************************
// command 13
// read timer
// **************************************************************
int BV4206::timerRead()
{ 
int rv;
  Wire.beginTransmission(_i2adr);
  Wire.send(0x13);  // command
  Wire.endTransmission();
  Wire.requestFrom(_i2adr, 1); // returns 1 byte
  rv=Wire.receive();
  Wire.endTransmission();
  return rv;
} 

// **************************************************************
// command 14
// timer source internal=1 or external=0 (def#ult)
// **************************************************************
void BV4206::timerSource(char source)
{ 
  Wire.beginTransmission(_i2adr);
  Wire.send(0x14);  // command
  if(source) Wire.send(TMR_INTERNAL);
  else Wire.send(TMR_EXTERNAL); 
  Wire.endTransmission();
} 

// **************************************************************
// command 15
// timer external increment control, 0=low-high, 1=high-low
// **************************************************************
void BV4206::timerIncrement(char control)
{ 
  Wire.beginTransmission(_i2adr);
  Wire.send(0x15);  // command
  if(control) Wire.send(TMR_LH);
  else Wire.send(TMR_HL); 
  Wire.endTransmission();
} 

// ==============================================================
// I2C PWM Section
// ==============================================================
// ==============================================================

// **************************************************************
// command 20
// PWM activate pwn ad set polarity 0=negative
// **************************************************************
void BV4206::pwmActivate(char polarity)
{ 
  Wire.beginTransmission(_i2adr);
  Wire.send(0x20);  // command
  if(polarity) Wire.send(PWM_PLUS);
  else Wire.send(PWM_MINUS); 
  Wire.endTransmission();
} 

// **************************************************************
// command 21
// PWM pulse width 0 to 408
// **************************************************************
void BV4206::pwmPeriod(int width)
{ 
  Wire.beginTransmission(_i2adr);
  Wire.send(0x21);  // command
  if(width > 408) width=408;
  if(width < 0 ) width=0;
  Wire.send((width >> 8) & 0xff); // high
  Wire.send(width & 0xff); // low 
  Wire.endTransmission();
} 




