/*
  BV4205 I2C LCD controller
  Copyright (c) 2011 Jim Spence.  All right reserved.
  www.byvac.com - see terms and conditions for using hardware
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/  

#include "WProgram.h"
#include "bv4208.h"
 
// **************************************************************
// **************************************************************
BV4208::BV4208(char i2adr):I2c_bv(i2adr)
{
  _i2adr=i2adr;  
}

// ==============================================================
// I2C LCD Controller
// ==============================================================
// ==============================================================

// **************************************************************
// command 1
// Send LCD Command
// **************************************************************
void BV4208::cmd(char value)
{ 
  // 
  Wire.beginTransmission(_i2adr);
  Wire.send(1);  // command
  Wire.send(value);
  Wire.endTransmission();
} 

// **************************************************************
// command 2
// send data
// **************************************************************
void BV4208::dat(char value)
{ 
  Wire.beginTransmission(_i2adr);
  Wire.send(2);  // command
  Wire.send(value);
  Wire.endTransmission();
} 

// **************************************************************
// command 3
// Back light on off, normally 0 id on 1 is off
// **************************************************************
void BV4208::bl(char value)
{ 
int rv;
  Wire.beginTransmission(_i2adr);
  Wire.send(3);  // command
  Wire.send(value);
  Wire.endTransmission();
} 

// **************************************************************
// command 4
// display on, blanks display using contrast pin
// **************************************************************
void BV4208::blank(char value)
{ 
int rv;
  Wire.beginTransmission(_i2adr);
  Wire.send(4);  // command
  Wire.send(value);
  Wire.endTransmission();
} 

// ==============================================================
// compound functions
// ==============================================================

// **************************************************************
// clear screen and home cursor
// **************************************************************
void BV4208::cls()
{ 
  Wire.beginTransmission(_i2adr);
  Wire.send(1);  // LCD command
  Wire.send(1); // it is normally this 
  Wire.endTransmission();
  delay(100);
} 

// **************************************************************
// Move
// move to a new cursor position given row and column
// Not all displays use these values:
// row 0 = 0x80
// row 1 = 0xc0
// row 2 = 0x94
// row 3 = 0xd4
// 0,0 is top left
// **************************************************************
void BV4208::position(char row, char col)
{ 
char new_rc;
  switch(row) {
    case 0: {new_rc=0x80+col; break;}
    case 1: {new_rc=0xc0+col; break;}
    case 2: {new_rc=0x94+col; break;}
    case 3: {new_rc=0xd4+col; break;}
    default: {new_rc=0xc0+col; break;} // row 1
  }
  Wire.beginTransmission(_i2adr);
  Wire.send(1);  // LCD command
  Wire.send(new_rc); // add row to col to get data to send 
  Wire.endTransmission();
} 

// **************************************************************
// Print
// Single charater - same as data
// **************************************************************
void BV4208::print(char value)
{ 
  dat(value);
} 

// **************************************************************
// Print
// String
// **************************************************************
void BV4208::print(char *s)
{ 
  Wire.beginTransmission(_i2adr);
  Wire.send(2);  // LCD data command
  while(*s) 
    Wire.send(*(s++));
  Wire.endTransmission();
} 

// **************************************************************
// Print
// integer
// **************************************************************
void BV4208::print(int value)
{ 
char b[15];
  itoa(value,b,10);
  print(b);
} 

// **************************************************************
// Print
// combines move and print string
// **************************************************************
void BV4208::print(char row, char col, char *s)
{
    position(row,col);
    print(s); 
} 

// **************************************************************
// defining custom charaters
// assumes that user defined charatcers start at 0x40 and
// custom numbers start at 0 and end at 7
// **************************************************************
void BV4208::custom(char number, 
                    char b1, char b2, char b3, char b4, 
                    char b5, char b6, char b7)
{
  // simple check
  if(number < 0) number=0;
  if(number > 7) number=7;
  Wire.beginTransmission(_i2adr);
  Wire.send(1);  // LCD command
  Wire.send(0x40+(8*number)); // number to define 
  Wire.endTransmission();
  // now send user data   
  Wire.beginTransmission(_i2adr);
  Wire.send(2);  // LCD data command 
  Wire.send(b1);Wire.send(b2);
  Wire.send(b3);Wire.send(b4);
  Wire.send(b5);Wire.send(b6);
  Wire.send(b7);
  Wire.endTransmission();
}


