/*
  BV4501 Serial Relay interface ByVac
  Copyright (c) 2011 Jim Spence.  All right reserved.
  www.byvac.com - see terms and conditions for using hardware
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
  Release Notes:

*/  

#include "WProgram.h"
#include "bv4501.h"


// ==============================================================
// BV4501 SECTION
// ==============================================================
// ==============================================================

// **************************************************************
// 
// **************************************************************
BV4501::BV4501(uint8_t rxPin, uint8_t txPin) : ASI(rxPin, txPin)
{
}

// **************************************************************
// Switches relay either on of off at a given time
// Input
// DEVadr   the ASI address e.g. 'b'
// relay    the relay 1 is A, 2 is B
// ron      1 to switch it on, 0 to switch it off
// time     time when switch happens 0 is now
// **************************************************************
void BV4501::rlySwitch(char DEVadr, char relay, char ron, int time)
{
char b[8];
    flush();
    putch(DEVadr);
    if(relay==1) {
        if(time==0) { // now
            if(ron=1) puts("a1\r");
            else puts("a0\r");
        }else {  // timed
            itoa(time,b,10); // get time as decimal string
            if(ron==1) puts("j"); 
            else puts("k");
            puts(b);
            puts("\r");
        }
        wait();        
    }
    if(relay==2) {
        if(time==0) { // now
            if(ron=1) puts("b1\r");
            else puts("b0\r");
        }else {  // timed
            itoa(time,b,10); // get time as decimal string
            if(ron==1) puts("m"); 
            else puts("n");
            puts(b);
            puts("\r");
        }
        wait();        
    }
}

// **************************************************************
// Get timer values for either a or b
// relay 1=A, 2=B
// **************************************************************
int BV4501::rlyTimer(char DEVadr, char relay)
{
char b[8];
    flush();
    putch(DEVadr);
    if(!(relay==1) || (relay==2)) return 0;
    if(relay==1) puts("l\r");
    else puts("o\r");
    wait(b, '>', 5);
    return atoi(b);
}

