/*
  BV4506 I2C Keypad
  Copyright (c) 2011 Jim Spence.  All right reserved.
  www.byvac.com - see terms and conditions for using hardware
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/  

#include "WProgram.h"
#include "bv4506.h"
 
// **************************************************************
// constructor
// **************************************************************
BV4506::BV4506(char i2adr):I2c_bv(i2adr)
{
  _i2adr=i2adr;  
}

// ==============================================================
// I2C Keypad
// ==============================================================
// ==============================================================

// **************************************************************
// command 1
//  # Number of keys in buffer
// **************************************************************
int BV4506::keys()
{ 
  Wire.beginTransmission(_i2adr);
  Wire.send(1);  // command
  Wire.endTransmission();
  return i2_16bit();
} 

// **************************************************************
// command 2
// clear buffer
// **************************************************************
void BV4506::clear()
{ 
  Wire.beginTransmission(_i2adr);
  Wire.send(2);  // command
  Wire.endTransmission();
} 

// **************************************************************
// command 3
// key down? 1 if so
// **************************************************************
int BV4506::down()
{ 
int rv;
  Wire.beginTransmission(_i2adr);
  Wire.send(3);  // command
  Wire.endTransmission();
  Wire.requestFrom(_i2adr, 1); // returns 1 byte
  rv=Wire.receive();
  Wire.endTransmission();
  return rv;
} 

// **************************************************************
// command 4
// get key from buffer
// **************************************************************
int BV4506::key()
{ 
int rv;
  Wire.beginTransmission(_i2adr);
  Wire.send(4);  // command
  Wire.endTransmission();
  Wire.requestFrom(_i2adr, 1); // returns 1 byte
  rv=Wire.receive();
  Wire.endTransmission();
  return rv;

} 

