/*
  VT100 library generic  ByVac
  Copyright (c) 2011 Jim Spence.  All right reserved.
  www.byvac.com - see terms and conditions for using hardware
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
  Release Notes:
  March 2011 released
*/  

#include "WProgram.h"
#include "vt100.h"

// **************************************************************
// normal constructor
// **************************************************************
VT100::VT100(uint8_t rxPin, uint8_t txPin) : BSerial(rxPin,txPin)
{

}

// **************************************************************
// set up baud rate, delay and ACK char
// set delay to 0 if none required and ack to 0 ditto
// **************************************************************
void VT100::begin(long rate, int cdelay, char ack)
{
char b[4];
    baud(rate);    // set baud rate
    _ack = ack;
    _delay = cdelay;
    // initialise Baud rate
    puts("\r");
    delay(300);
}

void VT100::begin(long rate, int cdelay)
{
char b[4];
    baud(rate);    // set baud rate
    _ack = 0;
    _delay = cdelay;
    // initialise Baud rate
    puts("\r");
    delay(300);
}
void VT100::begin(long rate)
{
char b[4];
    baud(rate);    // set baud rate
    _ack = 0;
    _delay = 0;
    // initialise Baud rate
    puts("\r");
    delay(300);
}
// ==============================================================
// SERIAL KEPAD
// ==============================================================
// ==============================================================

// **************************************************************
// VT100 reading information.
// No RTS available so either use a delay or ACK
// returns a string in buffer b
// use in the form xyz.cmds("\e[9c", b);
// **************************************************************
int VT100::cmds(char *cmd, char *b, int maxc)
{
#define TIMEOUT 900 // about 1 second
char *bp=b;
int c, timeout=TIMEOUT;
    flush();  // clear buffer
    puts(cmd);   // send command
    delay(_delay);  // should be 0 for ack
    // _ack set to 0 if not implemented
    if(_ack) {
        while(timeout) {
            if(buffer()==0) { 
                timeout--;
                continue;
            }
        c = getch();
        if(!maxc--) break;
        if( c == _ack) break;
        *(bp++)=c;
        timeout=TIMEOUT; // reset
        }
    } else { // no ack given, assume all in buffer
        while(buffer()!=0) {
            *(bp++) = getch();
        }
    }                        
    *bp=0;
}            

// **************************************************************
// VT100 reading information.
// No RTS available so either use a delay or ACK
// **************************************************************
int VT100::cmd(char *s)
{
#define TIMEOUT 900 // about 1 second
char b[5];
    cmds(s,b,4);
    // convert to a number and return
    return atoi(b);    
}            

